'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var axios = require('axios');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var axios__default = /*#__PURE__*/_interopDefaultLegacy(axios);

function _classCallCheck(instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
}

function _defineProperties(target, props) {
  for (var i = 0; i < props.length; i++) {
    var descriptor = props[i];
    descriptor.enumerable = descriptor.enumerable || false;
    descriptor.configurable = true;
    if ("value" in descriptor) descriptor.writable = true;
    Object.defineProperty(target, descriptor.key, descriptor);
  }
}

function _createClass(Constructor, protoProps, staticProps) {
  if (protoProps) _defineProperties(Constructor.prototype, protoProps);
  if (staticProps) _defineProperties(Constructor, staticProps);
  Object.defineProperty(Constructor, "prototype", {
    writable: false
  });
  return Constructor;
}

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function _inherits(subClass, superClass) {
  if (typeof superClass !== "function" && superClass !== null) {
    throw new TypeError("Super expression must either be null or a function");
  }

  Object.defineProperty(subClass, "prototype", {
    value: Object.create(superClass && superClass.prototype, {
      constructor: {
        value: subClass,
        writable: true,
        configurable: true
      }
    }),
    writable: false
  });
  if (superClass) _setPrototypeOf(subClass, superClass);
}

function _getPrototypeOf(o) {
  _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) {
    return o.__proto__ || Object.getPrototypeOf(o);
  };
  return _getPrototypeOf(o);
}

function _setPrototypeOf(o, p) {
  _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) {
    o.__proto__ = p;
    return o;
  };

  return _setPrototypeOf(o, p);
}

function _isNativeReflectConstruct() {
  if (typeof Reflect === "undefined" || !Reflect.construct) return false;
  if (Reflect.construct.sham) return false;
  if (typeof Proxy === "function") return true;

  try {
    Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {}));
    return true;
  } catch (e) {
    return false;
  }
}

function _assertThisInitialized(self) {
  if (self === void 0) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return self;
}

function _possibleConstructorReturn(self, call) {
  if (call && (typeof call === "object" || typeof call === "function")) {
    return call;
  } else if (call !== void 0) {
    throw new TypeError("Derived constructors may only return object or undefined");
  }

  return _assertThisInitialized(self);
}

function _createSuper(Derived) {
  var hasNativeReflectConstruct = _isNativeReflectConstruct();

  return function _createSuperInternal() {
    var Super = _getPrototypeOf(Derived),
        result;

    if (hasNativeReflectConstruct) {
      var NewTarget = _getPrototypeOf(this).constructor;

      result = Reflect.construct(Super, arguments, NewTarget);
    } else {
      result = Super.apply(this, arguments);
    }

    return _possibleConstructorReturn(this, result);
  };
}

function _unsupportedIterableToArray(o, minLen) {
  if (!o) return;
  if (typeof o === "string") return _arrayLikeToArray(o, minLen);
  var n = Object.prototype.toString.call(o).slice(8, -1);
  if (n === "Object" && o.constructor) n = o.constructor.name;
  if (n === "Map" || n === "Set") return Array.from(o);
  if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);
}

function _arrayLikeToArray(arr, len) {
  if (len == null || len > arr.length) len = arr.length;

  for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i];

  return arr2;
}

function _createForOfIteratorHelper(o, allowArrayLike) {
  var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"];

  if (!it) {
    if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") {
      if (it) o = it;
      var i = 0;

      var F = function () {};

      return {
        s: F,
        n: function () {
          if (i >= o.length) return {
            done: true
          };
          return {
            done: false,
            value: o[i++]
          };
        },
        e: function (e) {
          throw e;
        },
        f: F
      };
    }

    throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
  }

  var normalCompletion = true,
      didErr = false,
      err;
  return {
    s: function () {
      it = it.call(o);
    },
    n: function () {
      var step = it.next();
      normalCompletion = step.done;
      return step;
    },
    e: function (e) {
      didErr = true;
      err = e;
    },
    f: function () {
      try {
        if (!normalCompletion && it.return != null) it.return();
      } finally {
        if (didErr) throw err;
      }
    }
  };
}

class Connector {
    /**
     * Create a new class instance.
     */
    constructor(options) {
        /**
         * Default connector options.
         */
        this._defaultOptions = {
            auth: {
                headers: {},
            },
            authEndpoint: '/broadcasting/auth',
            broadcaster: 'pusher',
            csrfToken: null,
            host: null,
            key: null,
            namespace: 'App.Events',
        };
        this.setOptions(options);
        this.connect();
    }
    /**
     * Merge the custom options with the defaults.
     */
    setOptions(options) {
        this.options = Object.assign(this._defaultOptions, options);
        if (this.csrfToken()) {
            this.options.auth.headers['X-CSRF-TOKEN'] = this.csrfToken();
        }
        return options;
    }
    /**
     * Extract the CSRF token from the page.
     */
    csrfToken() {
        let selector;
        if (typeof window !== 'undefined' && window['Laravel'] && window['Laravel'].csrfToken) {
            return window['Laravel'].csrfToken;
        }
        else if (this.options.csrfToken) {
            return this.options.csrfToken;
        }
        else if (typeof document !== 'undefined' &&
            typeof document.querySelector === 'function' &&
            (selector = document.querySelector('meta[name="csrf-token"]'))) {
            return selector.getAttribute('content');
        }
        return null;
    }
}

var LOG_PREFIX = '[LE-AG-Websocket]';
var Websocket = /*#__PURE__*/function () {
  function Websocket(options) {
    _classCallCheck(this, Websocket);

    this.buffer = [];
    this.listeners = {};
    this.internalListeners = {};
    this.channelBacklog = [];
    this.closing = false;
    this.hasConnected = false;
    this.options = options;
    this.connect(this.options.host);
    return this;
  }

  _createClass(Websocket, [{
    key: "connect",
    value: function connect(host) {
      var _this = this;

      if (!host) {
        this.options.debug && console.error(LOG_PREFIX + "Cannont connect without host !");
        return;
      }

      this.options.debug && console.log(LOG_PREFIX + "Trying to connect to ".concat(host, "..."));
      this.websocket = new WebSocket(host);

      this.websocket.onerror = function () {
        if (!_this.hasConnected) {
          setTimeout(function () {
            _this.socketId = undefined;

            _this.connect(host);
          }, 3000);
        }
      };

      this.websocket.onopen = function () {
        _this.options.debug && console.log(LOG_PREFIX + ' Connected !');
        _this.hasConnected = true;

        _this.send({
          event: 'whoami'
        });

        while (_this.buffer.length) {
          var message = _this.buffer[0];

          _this.send(message);

          _this.buffer.splice(0, 1);
        } // Register events only once connected, or they won't be registered if connection failed/lost


        _this.websocket.onmessage = function (messageEvent) {
          var message = _this.parseMessage(messageEvent.data);

          _this.options.debug && console.log(LOG_PREFIX + ' onmessage', messageEvent.data);

          if (!message) {
            return;
          }

          if (message.channel) {
            _this.options.debug && console.log("".concat(LOG_PREFIX, " Received event ").concat(message.event, " on channel ").concat(message.channel));

            if (_this.listeners[message.channel] && _this.listeners[message.channel][message.event]) {
              _this.listeners[message.channel][message.event](message.data);
            }

            return;
          }

          if (_this.internalListeners[message.event]) {
            _this.internalListeners[message.event](message.data);
          }
        }; // send ping every 60 seconds to keep connection alive


        _this.pingInterval = setInterval(function () {
          if (_this.websocket.readyState === _this.websocket.OPEN) {
            _this.options.debug && console.log(LOG_PREFIX + ' Sending ping');

            _this.send({
              event: 'ping'
            });
          }
        }, 60 * 1000);
      };

      this.websocket.onclose = function () {
        _this.options.debug && console.info('Connection closed.');

        if (_this.closing) {
          return;
        }

        _this.hasConnected = false;
        _this.options.debug && console.info('Connection lost, reconnecting...');
        setTimeout(function () {
          _this.socketId = undefined;

          _this.connect(host);
        }, 1000);
      };

      this.on('whoami', function (_ref) {
        var socketId = _ref.socket_id;
        _this.socketId = socketId;
        _this.options.debug && console.log("".concat(LOG_PREFIX, " Just set socketId to ").concat(socketId)); // Handle the backlog and don't empty it, we'll need it if we lose connection

        var channel;

        var _iterator = _createForOfIteratorHelper(_this.channelBacklog),
            _step;

        try {
          for (_iterator.s(); !(_step = _iterator.n()).done;) {
            channel = _step.value;

            _this.actuallySubscribe(channel);
          }
        } catch (err) {
          _iterator.e(err);
        } finally {
          _iterator.f();
        }
      });
    }
  }, {
    key: "parseMessage",
    value: function parseMessage(body) {
      try {
        return JSON.parse(body);
      } catch (error) {
        this.options.debug && console.error(error);
        return undefined;
      }
    }
  }, {
    key: "getSocketId",
    value: function getSocketId() {
      return this.socketId;
    }
  }, {
    key: "socketIsReady",
    value: function socketIsReady() {
      return this.websocket.readyState === this.websocket.OPEN;
    }
  }, {
    key: "send",
    value: function send(message) {
      if (this.socketIsReady()) {
        this.websocket.send(JSON.stringify(message));
        return;
      }

      this.buffer.push(message);
    }
  }, {
    key: "close",
    value: function close() {
      this.closing = true;
      this.internalListeners = {};
      clearInterval(this.pingInterval);
      this.pingInterval = undefined;
      this.websocket.close();
    }
  }, {
    key: "subscribe",
    value: function subscribe(channel) {
      if (this.getSocketId()) {
        this.actuallySubscribe(channel);
      } else {
        this.options.debug && console.log("".concat(LOG_PREFIX, " subscribe - push channel backlog for channel ").concat(channel.name));
        this.channelBacklog.push(channel);
      }
    }
  }, {
    key: "actuallySubscribe",
    value: function actuallySubscribe(channel) {
      var _this2 = this;

      if (channel.name.startsWith('private-') || channel.name.startsWith('presence-')) {
        this.options.debug && console.log("".concat(LOG_PREFIX, " Sending auth request for channel ").concat(channel.name));

        if (this.options.bearerToken) {
          this.options.auth.headers['Authorization'] = 'Bearer ' + this.options.bearerToken;
        }

        axios__default['default'].post(this.options.authEndpoint, {
          socket_id: this.getSocketId(),
          channel_name: channel.name
        }, {
          headers: this.options.auth.headers || {}
        }).then(function (response) {
          _this2.options.debug && console.log("".concat(LOG_PREFIX, " Subscribing to private channel ").concat(channel.name));

          _this2.send({
            event: 'subscribe',
            data: _extends({
              channel: channel.name
            }, response.data)
          });
        })["catch"](function (error) {
          _this2.options.debug && console.log("".concat(LOG_PREFIX, " Auth request for channel ").concat(channel.name, " failed"));
          _this2.options.debug && console.error(error);
        });
      } else {
        this.options.debug && console.log("".concat(LOG_PREFIX, " Subscribing to channel ").concat(channel.name));
        this.send({
          event: 'subscribe',
          data: {
            channel: channel.name
          }
        });
      }
    }
  }, {
    key: "unsubscribe",
    value: function unsubscribe(channel) {
      this.options.debug && console.log("".concat(LOG_PREFIX, " unsubscribe for channel ").concat(channel.name));
      this.send({
        event: 'unsubscribe',
        data: {
          channel: channel.name
        }
      });

      if (this.listeners[channel.name]) {
        delete this.listeners[channel.name];
      }
    }
  }, {
    key: "on",
    value: function on(event) {
      var callback = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;
      this.options.debug && console.log("".concat(LOG_PREFIX, " on event ").concat(event, " ..."));
      this.internalListeners[event] = callback;
    }
  }, {
    key: "bind",
    value: function bind(channel, event, callback) {
      this.options.debug && console.log("".concat(LOG_PREFIX, " bind event ").concat(event, " for channel ").concat(channel.name, " ..."));

      if (!this.listeners[channel.name]) {
        this.listeners[channel.name] = {};
      }

      this.listeners[channel.name][event] = callback;
    }
  }, {
    key: "unbindEvent",
    value: function unbindEvent(channel, event) {
      var callback = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : null;
      this.options.debug && console.log("".concat(LOG_PREFIX, " unbind event ").concat(event, " for channel ").concat(channel.name, " ..."));

      if (this.internalListeners[event] && (callback === null || this.internalListeners[event] === callback)) {
        delete this.internalListeners[event];
      }
    }
  }]);

  return Websocket;
}();

/**
 * Event name formatter
 */
class EventFormatter {
    /**
     * Create a new class instance.
     */
    constructor(namespace) {
        this.setNamespace(namespace);
    }
    /**
     * Format the given event name.
     */
    format(event) {
        if (event.charAt(0) === '.' || event.charAt(0) === '\\') {
            return event.substr(1);
        }
        else if (this.namespace) {
            event = this.namespace + '.' + event;
        }
        return event.replace(/\./g, '\\');
    }
    /**
     * Set the event namespace.
     */
    setNamespace(value) {
        this.namespace = value;
    }
}

/**
 * This class represents a basic channel.
 */
class Channel {
    /**
     * Listen for a whisper event on the channel instance.
     */
    listenForWhisper(event, callback) {
        return this.listen('.client-' + event, callback);
    }
    /**
     * Listen for an event on the channel instance.
     */
    notification(callback) {
        return this.listen('.Illuminate\\Notifications\\Events\\BroadcastNotificationCreated', callback);
    }
    /**
     * Stop listening for a whisper event on the channel instance.
     */
    stopListeningForWhisper(event, callback) {
        return this.stopListening('.client-' + event, callback);
    }
}

var LOG_PREFIX$1 = '[LE-AG-Channel]';
/**
 * This class represents a Pusher channel.
 */

var Channel$1 = /*#__PURE__*/function (_BaseChannel) {
  _inherits(Channel, _BaseChannel);

  var _super = _createSuper(Channel);

  /**
   * Create a new class instance.
   */
  function Channel(socket, name, options) {
    var _this;

    _classCallCheck(this, Channel);

    _this = _super.call(this);
    _this.name = name;
    _this.socket = socket;
    _this.options = options;
    _this.eventFormatter = new EventFormatter(_this.options["namespace"]);

    _this.subscribe();

    return _this;
  }
  /**
   * Subscribe to a Pusher channel.
   */


  _createClass(Channel, [{
    key: "subscribe",
    value: function subscribe() {
      this.options["debug"] && console.log("".concat(LOG_PREFIX$1, " subscribe for channel ").concat(this.name, " ..."));
      this.socket.subscribe(this);
    }
    /**
     * Unsubscribe from a Pusher channel.
     */

  }, {
    key: "unsubscribe",
    value: function unsubscribe() {
      this.options["debug"] && console.log("".concat(LOG_PREFIX$1, " unsubscribe for channel ").concat(this.name, " ..."));
      this.socket.unsubscribe(this);
    }
    /**
     * Listen for an event on the channel instance.
     */

  }, {
    key: "listen",
    value: function listen(event, callback) {
      this.options["debug"] && console.log("".concat(LOG_PREFIX$1, " listen to ").concat(event, " for channel ").concat(this.name, " ..."));
      this.on(this.eventFormatter.format(event), callback);
      return this;
    }
    /**
     * Stop listening for an event on the channel instance.
     */

  }, {
    key: "stopListening",
    value: function stopListening(event, callback) {
      this.options["debug"] && console.log("".concat(LOG_PREFIX$1, " stop listening to ").concat(event, " for channel ").concat(this.name, " ..."));
      this.socket.unbindEvent(this, event, callback);
      return this;
    }
    /**
     * Register a callback to be called anytime a subscription succeeds.
     */

  }, {
    key: "subscribed",
    value: function subscribed(callback) {
      this.options["debug"] && console.log("".concat(LOG_PREFIX$1, " subscribed for channel ").concat(this.name, " ..."));
      this.on('subscription_succeeded', function () {
        callback();
      });
      return this;
    }
    /**
     * Register a callback to be called anytime a subscription error occurs.
     */

  }, {
    key: "error",
    value: function error(callback) {
      this.options["debug"] && console.log("".concat(LOG_PREFIX$1, " error for channel ").concat(this.name, " ..."));
      this.on('error', function (status) {
        callback(status);
      });
      return this;
    }
    /**
     * Bind a channel to an event.
     */

  }, {
    key: "on",
    value: function on(event, callback) {
      this.options["debug"] && console.log("".concat(LOG_PREFIX$1, " on ").concat(event, " for channel ").concat(this.name, " ..."));
      this.socket.bind(this, event, callback);
      return this;
    }
  }, {
    key: "whisper",
    value: function whisper(event, data) {
      var channel = this.name;
      var formattedEvent = "client-" + event;
      this.socket.send({
        "event": formattedEvent,
        data: data,
        channel: channel
      });
      return this;
    }
  }, {
    key: "here",
    value: function here(callback) {
      // TODO: implement
      return this;
    }
    /**
     * Listen for someone joining the channel.
     */

  }, {
    key: "joining",
    value: function joining(callback) {
      // TODO: implement
      return this;
    }
    /**
     * Listen for someone leaving the channel.
     */

  }, {
    key: "leaving",
    value: function leaving(callback) {
      // TODO: implement
      return this;
    }
  }]);

  return Channel;
}(Channel);

var broadcaster = function broadcaster(options) {
  return new Connector$1(options);
};
var LOG_PREFIX$2 = '[LE-AG-Connector]';
var Connector$1 = /*#__PURE__*/function (_BaseConnector) {
  _inherits(Connector, _BaseConnector);

  var _super = _createSuper(Connector);

  /**
   * Create a new class instance.
   */
  function Connector(options) {
    var _this;

    _classCallCheck(this, Connector);

    _this = _super.call(this, options);
    /**
     * All of the subscribed channel names.
     */

    _this.channels = {};
    return _this;
  }
  /**
   * Create a fresh Socket.io connection.
   */


  _createClass(Connector, [{
    key: "connect",
    value: function connect() {
      this.options.debug && console.log(LOG_PREFIX$2 + 'Connect ...');
      this.socket = new Websocket(this.options);
      return; //
      // this.socket.on('reconnect', () => {
      //     Object.values(this.channels).forEach((channel) => {
      //         channel.subscribe();
      //     });
      // });
      //
      // return this.socket;
    }
    /**
     * Get a channel instance by name.
     */

  }, {
    key: "channel",
    value: function channel(name) {
      if (!this.channels[name]) {
        this.channels[name] = new Channel$1(this.socket, name, this.options);
      }

      return this.channels[name];
    }
    /**
     * Get a private channel instance by name.
     */

  }, {
    key: "privateChannel",
    value: function privateChannel(name) {
      if (!this.channels['private-' + name]) {
        this.channels['private-' + name] = new Channel$1(this.socket, 'private-' + name, this.options);
      }

      return this.channels['private-' + name];
    }
    /**
     * Get a presence channel instance by name.
     */

  }, {
    key: "presenceChannel",
    value: function presenceChannel(name) {
      if (!this.channels['presence-' + name]) {
        this.channels['presence-' + name] = new Channel$1(this.socket, 'presence-' + name, this.options);
      }

      return this.channels['presence-' + name];
    }
    /**
     * Leave the given channel, as well as its private and presence variants.
     */

  }, {
    key: "leave",
    value: function leave(name) {
      var _this2 = this;

      var channels = [name, 'private-' + name, 'presence-' + name];
      channels.forEach(function (name) {
        _this2.leaveChannel(name);
      });
    }
    /**
     * Leave the given channel.
     */

  }, {
    key: "leaveChannel",
    value: function leaveChannel(name) {
      if (this.channels[name]) {
        this.channels[name].unsubscribe();
        delete this.channels[name];
      }
    }
    /**
     * Get the socket ID for the connection.
     */

  }, {
    key: "socketId",
    value: function socketId() {
      return this.socket.getSocketId();
    }
    /**
     * Disconnect socket connection.
     */

  }, {
    key: "disconnect",
    value: function disconnect() {
      this.options.debug && console.log(LOG_PREFIX$2 + 'Disconnect ...');
      this.socket.close();
    }
  }]);

  return Connector;
}(Connector);

exports.Connector = Connector$1;
exports.broadcaster = broadcaster;
